/*
## 2006 license information 
##
## The code in this file actually is taken from Daniel Bernsteins Primegen
## library functions (version 0.97, see http://cr.yp.to/primegen.html). 
## Only little coding modifications have been done. 
## 
## This file is provided to be used by several Octave functions. 
## 
## Since there are no special license terms in D. Bernsteins original 
## Primegen distribution it is assumed that the code can freely be used.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

$Revision: 1.1 $
$Date: 2006/06/11 01:14:09 $
$RCSfile: prime-gen.h,v $

Torsten Finke   <fi@igh-essen.com>
*/

typedef unsigned long ulong;
typedef unsigned long long udlong;
typedef long long idlong;

#define PRIMEGEN_WORDS 2048

typedef struct {
    ulong buf[16][PRIMEGEN_WORDS];
    udlong p[512]; /* p[num-1] ... p[0], in that order */
    int num;
    int pos; /* next entry to use in buf; WORDS to restart */
    udlong base;
    udlong L;
} primegen;

void primegen_sieve(primegen *pg);

/*
B is 32 times X.
Total memory use for one generator is 2B bytes = 64X bytes.
Covers primes in an interval of length 1920X.
Working set size for one generator is B bits = 4X bytes.

Speedup by a factor of 2 or 3 for L1 cache instead of L2 cache.
Slowdown by a factor of roughly n for primes past (nB)^2.

Possible choices of X:
  2002 to fit inside an 8K L1 cache (e.g., Pentium).
  4004 to fit inside a 16K L1 cache (e.g., Pentium II).
  64064 to fit inside a 256K L2 cache.

There are various word-size limits on X; 1000000 should still be okay.
*/

#define B32 PRIMEGEN_WORDS
#define B (PRIMEGEN_WORDS * 32)

